/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
 
package de.bsvrz.vew.engvew.verwaltung;

import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.engvew.engstelle.Situation;
import de.bsvrz.vew.engvew.engstelle.Stau;

/**
 * Klasse zur Verwaltung von Engstellen in VerkehrsModellNetzen.<br>
 * Beim Instanziieren eines Klassenobjekts werden aus einem Objekt vom Typ 
 * typ.netz rekursiv die Staus aus allen enthaltenen VerkehrsModellNetzen
 * zur Verwaltung eingelesen. 
 * Die Mengen der Staus werden fr alle VerkehrsModellNetze berwacht, 
 * desweiteren wird die Invalidierung (Lschung) von Staus berwacht.
 * 
 * @author Dambach Werke GmbH
 * @author Stefan Sans
 * @version $Revision: 1.1 $ / $Date: 2008/10/31 13:14:30 $ / ($Author: Sans $)
 */
public class VerwaltungStaus extends VerwaltungSituationen 
{  
 
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Verbindung zum Datenverteiler
   */
  private ClientDavInterface _connection;
    
  /**
   * Erzeuge eine Instanz vom Typ <code>VerwaltungStaus</code>
   * 
   * @param connection die Datenverteiler-Verbindung
   * @param netz das (Wurzel-) Objekt vom Typ typ.netz
   */
  public VerwaltungStaus(ClientDavInterface connection, 
      ConfigurationObject netz)
  {
    _connection = connection;
    if (!initialisiereNetz(netz))
    {
      throw new IllegalArgumentException("Das bergebene Netz " + netz.getName() +
      " enthlt KEINE VerkehrsModellNetze");
    }
  }
  
  @Override
  protected void initialisiereVerkehrsModellNetz(ConfigurationObject vmn)
  {
    initialisiereMenge(vmn, "Staus");
  }
    
  @Override
  protected Stau erzeugeSituation(SystemObject objekt)
  {
    Stau stau = null; 
    
    if (objekt.isOfType("typ.stau"))
    {
      stau = new Stau(_connection, objekt);
      _debug.fine("Stau erzeugt fr " + objekt);
      stau.anmelden();
    }
    else
    {
      _debug.warning("Objekt mit PID " + objekt.getPid() + 
      " ist vom nicht untersttztem Typ " + objekt.getType());
    }
    
    return stau;
  }  
  
  @Override
  protected void bereinigeSituation(Situation situation)
  {
    if (situation instanceof Stau)
    {
      _debug.fine("Bereinige Stau fr " + situation.getObjekt());
      ((Stau)situation).abmelden();
      
      for (BereinigungsListener listener: _bereinigungsDaten.keySet())
      {
        for (SystemObject stau: _bereinigungsDaten.get(listener))
        {
          if (stau == situation)
          {
            listener.bereinige(stau);
          }
        }
      }
    }
  }
  
  /**
   * @return Die Menge der zu benachrichtigenden <code>BereinigungsListener</code>, wenn ein 
   * Stau entfernt wird
   */
  Map<BereinigungsListener, Set<SystemObject>> getBereinigungsDaten()
  {
    return _bereinigungsDaten;
  }

  /**
   * Die Menge der zu benachrichtigenden <code>BereinigungsListener</code>, wenn ein 
   * Stau entfernt wird
   */
  private Map<BereinigungsListener, Set<SystemObject>> _bereinigungsDaten =
    new LinkedHashMap<BereinigungsListener, Set<SystemObject>>();

  
  /**
   * Anmelden eines <code>BereinigungsListener</code> fr einen bestimmten Stau
   * 
   * @param listener der <code>BereinigungsListener</code>, der benachrichtigt werden soll
   * @param stau der stau, der berwacht werden soll
   */
  public void anmeldenBereinigung(BereinigungsListener listener, SystemObject stau)
  {
    Set<SystemObject> staus = _bereinigungsDaten.get(listener);
    if (staus == null)
    {
      staus = new LinkedHashSet<SystemObject>();
      _bereinigungsDaten.put(listener, staus);
    }
    staus.add(stau);
  }
  
  /**
   * Abmelden eines <code>BereinigungsListener</code> fr einen bestimmten Stau
   * 
   * @param listener der <code>BereinigungsListener</code>, der nicht mehr benachrichtigt 
   * werden soll
   * @param stau der stau, der nicht mehr berwacht werden soll
   */
  public void abmeldenBereinigung(BereinigungsListener listener, SystemObject stau)
  {
    Set<SystemObject> staus = _bereinigungsDaten.get(listener);
    if (staus != null)
    {
      staus.remove(stau);
      if (staus.isEmpty())
      {
        _bereinigungsDaten.remove(listener);
      }
    }
  }
  
  /**
   * Schnittstelle, die von Klassen implementiert wird, deren 
   * Instanzen von der Bereinigung von Staus benachichtigt werden
   * 
   * @author Stefan Sans
   */
  public static interface BereinigungsListener
  {
    /**
     * Diese Methode wird aufgerufen, wenn in der Stauverwaltung
     * das bergebene Stau-Objekt bereinigt wurde.
     * 
     * @param stau das Stau-Objekt, das bereinigt wurde
     */
    public void bereinige(SystemObject stau);
  }
  
}
